package SPVM::Go::Channel::Wait;



1;

=head1 Name

SPVM::Go::Channel::Wait - Short Description

=head1 Description

Go::Channel::Wait class in L<SPVM> has methods for someting.

=head1 Usage

  use Go::Channel::Wait;

=head1 Fields



=head1 Class Methods



=head1 Instance Methods




=head1 Repository



=head1 Author

Yuki Kimoto C<kimoto.yuki@gmail.com>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

