package SPVM::Go::Select::Case;



1;

=head1 Name

SPVM::Go::Select::Case - Selected Cases

=head1 Description

The Go::Select::Case class in L<SPVM> represents selected cases.

=head1 Usage

  use Go::Select::Case;

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

