package SPVM::Go::Coroutine;



1;

=head1 Name

SPVM::Go::Coroutine - Coroutines

=head1 Description

The Go::Coroutine class in L<SPVM> has methods to manipulate coroutines.

=head1 Usage

  use Go::Coroutine;

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

